static const char file_id[] = "DataBlock.cc";
/**************************************************************************
Author:  Eric K. Pauer

$Id: DataBlock.cc,v 1.3 1995/07/05 19:06:14 epauer Exp $

This work was performed by Sanders, a Lockheed Martin Company, as part
of the Sanders RASSP program under contract N00014-93-C-2172 to the
Naval Research Laboratory, 4555 Overlook Avenue, SW, Washington, DC
20375-5326. The Sponsoring Agency is: Advanced Research Projects
Agency, Electronic System Technology Office, 3701 North Fairfax
Drive, Arlington, VA 22203-1714. The Sanders RASSP team consists of
Sanders, Motorola, Hughes, and ISX.

Methods for class DataBlock
**************************************************************************/
#ifdef __GNUG__
#pragma implementation
#endif
#include "DataBlock.h"
#include "string.h"

// constructor
DataBlock::DataBlock(int dataSize, const char *dataRepresentation, int processor, int functionBlock, int portCount, int *portList) : 
  dataSize_(dataSize), processor_(processor), functionBlock_(functionBlock), portCount_(portCount)
  {
  strcpy(dataRepresentation_, dataRepresentation);
  for (int i = 0; i < portCount_; i++)
    portList_[i] = portList[i];
  }

// constructor
DataBlock::DataBlock(int dataSize, const char *dataRepresentation, int processor, int functionBlock, int port) : 
  dataSize_(dataSize), processor_(processor), functionBlock_(functionBlock), portCount_(1)
  {
  strcpy(dataRepresentation_, dataRepresentation);
  portList_[0] = port;
  }

// copy constructor
DataBlock::DataBlock(const DataBlock& src) 
  { 
  dataSize_ = src.dataSize_;
  strcpy(dataRepresentation_, src.dataRepresentation_);
  processor_ = src.processor_;
  functionBlock_ = src.functionBlock_;
  portCount_ = src.portCount_;
  for (int i = 0; i < portCount_; i++)
    portList_[i] = src.portList_[i];
  }

Message* DataBlock::clone() const
  {
  LOG_NEW;
  return new DataBlock(*this);
  }

StringList DataBlock::print() const
  {
  StringList out = "Message class <";
  out += dataType();
  out += ">:  Size = ";
  out += dataSize();
  out += ", Type = ";
  out += dataRepresentation();
  out += ", Processor = ";
  out += processor();
  out += ", FunctionBlock = ";
  out += functionBlock();
  out += ", PortList = ";
  for (int i = 0; i < portCount_; i++)
    {
    out += portList_[i];
    out += " ";
    }
  return out;
  }

int DataBlock::dataSize() const 
  { 
  return dataSize_;
  }

const char* DataBlock::dataRepresentation() const
  {
  return dataRepresentation_;
  }

int DataBlock::processor() const
  {
  return processor_;
  }

int DataBlock::functionBlock() const
  {
  return functionBlock_;
  }

int DataBlock::portCount() const
  {
  return portCount_;
  }

int DataBlock::portList(int i) const
  {
  return portList_[i];
  }

void DataBlock::setDataSize(int dataSize)
  {
  dataSize_ = dataSize;
  }

void DataBlock::setDataRepresentation(const char *dataRepresentation)
  {
  strcpy(dataRepresentation_, dataRepresentation);
  }

void DataBlock::setProcessor(int processor)
  {
  processor_ = processor;
  }

void DataBlock::setFunctionBlock(int functionBlock)
  {
  functionBlock_ = functionBlock;
  }

void DataBlock::setPortCount(int portCount)
  {
  portCount_ = portCount;
  }

void DataBlock::setPortList(int index, int port)
  {
  portList_[index] = port;
  }


const char* DataBlock::dataType() const
  {
  return "DataBlock";
  }

ISA_FUNC(DataBlock,Message);
