static const char file_id[] = "ResourceBlock.cc";
/**************************************************************************
Author:  Eric K. Pauer

$Id: ResourceBlock.cc,v 1.2 1995/07/18 10:54:37 epauer Exp $

This work was performed by Sanders, a Lockheed Martin Company, as part
of the Sanders RASSP program under contract N00014-93-C-2172 to the
Naval Research Laboratory, 4555 Overlook Avenue, SW, Washington, DC
20375-5326. The Sponsoring Agency is: Advanced Research Projects
Agency, Electronic System Technology Office, 3701 North Fairfax
Drive, Arlington, VA 22203-1714. The Sanders RASSP team consists of
Sanders, Motorola, Hughes, and ISX.

Methods for class ResourceBlock
**************************************************************************/
#ifdef __GNUG__
#pragma implementation
#endif
#include "ResourceBlock.h"
#include "string.h"

// constructor
ResourceBlock::ResourceBlock(const char *functionName, int functionNumber, int cycles, int memory, int vectorSize) 
: functionNumber_(functionNumber), cycles_(cycles), memory_(memory), vectorSize_(vectorSize)
  {
  strcpy(functionName_, functionName);
  }

// copy constructor
ResourceBlock::ResourceBlock(const ResourceBlock& src) 
  {
  cycles_ = src.cycles_;
  memory_ = src.memory_;
  vectorSize_ = src.vectorSize_;
  }

Message* ResourceBlock::clone() const
  {
  LOG_NEW;
  return new ResourceBlock(*this);
  }

StringList ResourceBlock::print() const
  {
  StringList out = "Message class <";
  out += dataType();
  out += ">:  Cycles = ";
  out += cycles();
  out += ", Memory = ";
  out += memory();
  out += ", VectorSize = ";
  out += vectorSize();
  return out;
  }

int ResourceBlock::cycles() const 
  { 
  return cycles_;
  }

int ResourceBlock::memory() const
  {
  return memory_;
  }

int ResourceBlock::vectorSize() const
  {
  return vectorSize_;
  }

const char* ResourceBlock::functionName() const
  {
  return functionName_;
  }

int ResourceBlock::functionNumber() const
  {
  return functionNumber_;
  }

void ResourceBlock::setCycles(int cycles)
  {
  cycles_ = cycles;
  }

void ResourceBlock::setMemory(int memory)
  {
  memory_ = memory;
  }

void ResourceBlock::setVectorSize(int vectorSize)
  {
  vectorSize_ = vectorSize;
  }

void ResourceBlock::setFunctionName(const char* functionName)
  {
  strcpy(functionName_, functionName);
  }

void ResourceBlock::setFunctionNumber(int functionNumber)
  {
  functionNumber_ = functionNumber;
  }

const char* ResourceBlock::dataType() const
  {
  return "ResourceBlock";
  }

ISA_FUNC(ResourceBlock,Message);

