static const char file_id[] = "../../../../../src/contrib/atrade/de/stars/DEBus.pl";
// .cc file generated from ../../../../../src/contrib/atrade/de/stars/DEBus.pl by ptlang
/*
Copyright (c) 1995 Sanders, a Lockheed Martin Company
All rights reserved.
 */

#ifdef __GNUG__
#pragma implementation
#endif

#include "DEBus.h"
#include "DEProcessor.h"

const char *star_nm_DEBus = "DEBus";

const char* DEBus :: className() const {return star_nm_DEBus;}

ISA_FUNC(DEBus,DEStar);

Block* DEBus :: makeNew() const { LOG_NEW; return new DEBus;}

DEBus::DEBus ()
{
	setDescriptor("Models a bus by coordinating resources (bandwidth and users)");
	addPort(inputPort.setPort("inputPort",this,MESSAGE));
	addPort(outputPort.setPort("outputPort",this,MESSAGE));
	addState(partNumber.setState("partNumber",this,"","Type of bus",
# line 34 "../../../../../src/contrib/atrade/de/stars/DEBus.pl"
A_CONSTANT | A_SETTABLE));
	addState(busNumber.setState("busNumber",this,"0","Unique id of the bus",
# line 41 "../../../../../src/contrib/atrade/de/stars/DEBus.pl"
A_CONSTANT | A_SETTABLE));
	addState(bandwidth.setState("bandwidth",this,"1000000","Clock rate of the processor in words per second",
# line 48 "../../../../../src/contrib/atrade/de/stars/DEBus.pl"
A_CONSTANT | A_SETTABLE));
	addState(busWidth.setState("busWidth",this,"32","Width of bus in bits",
# line 55 "../../../../../src/contrib/atrade/de/stars/DEBus.pl"
A_CONSTANT | A_SETTABLE));
	addState(simultaneousUsers.setState("simultaneousUsers",this,"1","Number of users who can access the bus simultaneously",
# line 62 "../../../../../src/contrib/atrade/de/stars/DEBus.pl"
A_CONSTANT | A_SETTABLE));

# line 67 "../../../../../src/contrib/atrade/de/stars/DEBus.pl"
delayType = TRUE;
}

void DEBus::begin() {
# line 81 "../../../../../src/contrib/atrade/de/stars/DEBus.pl"
busy = new double[simultaneousUsers];
	  for (i = 0; i < simultaneousUsers; i++)
            busy[i] = double(0);

     	  clog.open(DEProcessor::logname, ios::app);
          clog << this->fullName() << ": ";
	  clog << "inputPorts = " << inputPort.numberPorts() << ", outputPorts = " << outputPort.numberPorts() << endl;
	  clog.close();
}

void DEBus::go() {
# line 92 "../../../../../src/contrib/atrade/de/stars/DEBus.pl"
// check if data has arrived on the internalInput
          InDEMPHIter sourceIter(inputPort);
	  for (i = 0; i < inputPort.numberPorts(); i++)
	    {
	    source = sourceIter++;
	    if (source->dataNew)
	      {
	      Envelope pkt;
	      (source->get()).getMessage(pkt);
	      if (!pkt.typeCheck("DataBlock"))
	        {
	        cerr << "Fatal Error: " << this->fullName() << ", typeError(DataBlock)" << endl;
	        Error::abortRun(*this, pkt.typeError("DataBlock"));
	        return;
	        }
              const DataBlock* dataBlockPtr = (const DataBlock *)pkt.myData();
	      delay = double(int(dataBlockPtr->dataSize()) * 4) / (double(busWidth / 8) * double(bandwidth));
     	      clog.open(DEProcessor::logname, ios::app);
	      clog << this->fullName() << ", a data block of " << dataBlockPtr->dataSize() << " samples of ";
	      clog << dataBlockPtr->dataRepresentation() << " received on inputPort " << i << " at time ";
	      clog << arrivalTime << ", transfer time of " << delay << " calculated" << endl;
	      clog.close();
	      wait = 1;
	      earliestAvailable = busy[0];
	      earliestAvailableIndex = 0;
	      // check for available time slot
	      for (j = 0; j < simultaneousUsers; j++)
		{
		if (arrivalTime > busy[j])
		  {
		  // resource available, reserve for appropriate time slot
		  busy[j] = arrivalTime + delay;
		  completionTime = busy[j];
		  wait = 0;
		  break;
		  }
		else
		  // resource not available, keep track of earilest available time slot
		  if (busy[j] < earliestAvailable)
		    {
		    earliestAvailable = busy[j];
	            earliestAvailableIndex = j;
		    }
		}
	      // if limit has been reached, utilize first available time slot
              if (wait)
		{
		busy[earliestAvailableIndex] += delay;
		completionTime = busy[earliestAvailableIndex];
		}
	      clog.open(DEProcessor::logname, ios::app);
	      clog << this->fullName() << ", transmission of a data block of " << dataBlockPtr->dataSize();
	      clog << " samples of " << dataBlockPtr->dataRepresentation() << " to " << outputPort.numberPorts();
	      clog << " processors on this bus completed at time " << completionTime << endl;
	      clog.close();
	      ofstream outFile(DEProcessor::filename, ios::app);
	      if (!outFile)
	        {
	        cerr << "Fatal Error: " << this->fullName() << ", unable to open output file " << DEProcessor::filename << endl;
	        Error::abortRun(*this,"Unable to open output file.");
	        }
	      outFile << "B" << busNumber << " " << dataBlockPtr->processor() << " " << (completionTime - delay) << " " << completionTime << endl;
	      outFile.close();
	      // send data to all outputs
              OutDEMPHIter destIter(outputPort);
	      for (j = 0; j < outputPort.numberPorts(); j++)
		{
		dest = destIter++;
		dest->put(completionTime) << pkt;
		}
	      }
	    }
}

// prototype instance for known block list
static DEBus proto;
static RegisterBlock registerBlock(proto,"Bus");
