defstar {
	name {Function}
	domain {DE}
	version { $Revision: 1.7 $ $Date: 1995/07/19 18:45:45 $ }
	author { E. K. Pauer }
	copyright {
Copyright (c) 1995 Sanders, a Lockheed Martin Company
All rights reserved.
	}
	acknowledge {
This work was performed by Sanders, a Lockheed Martin Company, as part
of the Sanders RASSP program under contract N00014-93-C-2172 to the
Naval Research Laboratory, 4555 Overlook Avenue, SW, Washington, DC
20375-5326. The Sponsoring Agency is: Advanced Research Projects
Agency, Electronic System Technology Office, 3701 North Fairfax
Drive, Arlington, VA 22203-1714. The Sanders RASSP team consists of
Sanders, Motorola, Hughes, and ISX.
}
	location { $PTOLEMY/src/contrib/atrade/de/stars }
	desc { Models the performance of an SDF star based on cycles and memory needed. }
	inmulti {
		name {inputPort}
		type {message}
	}
	outmulti {
		name {outputPort}
		type {message}
	}
	input {
		name {acknowledge}
		type {message}
	}
	output {
		name {request}
		type {message}
	}
	defstate {
		name {functionName}
		type {string}
		default {""}
		desc { Name of function implemented }
		attributes { A_CONSTANT | A_SETTABLE }
	}
	defstate {
		name {functionNumber}
		type {int}
		default {0}
		desc { Unique id of the functional block }
	}
	defstate {
		name {inputSamples}
		type {intarray}
		default {1}
		desc { DataBlocks needed on each input for star to fire }
		attributes { A_CONSTANT | A_SETTABLE }
	}
	defstate {
		name {outputSamples}
		type {intarray}
		default {1}
		desc { DataBlocks produced on each output per each firing }
		attributes { A_CONSTANT | A_SETTABLE }
	}
	defstate {
		name {inputDataType}
		type {stringarray}
		default {"FLOAT"}
		desc { Type of data accepted by each inputPort }
		attributes { A_CONSTANT | A_SETTABLE }
	}
	defstate {
		name {outputDataType}
		type {stringarray}
		default {"FLOAT"}
		desc { Type of data produced by each outputPort }
		attributes { A_CONSTANT | A_SETTABLE }
	}
	defstate {
		name {processingCostOverhead}
		type {int}
		default {15}
		desc { number of overhead CPU cycles needed to fire }
		attributes { A_CONSTANT | A_SETTABLE }
	}
	defstate {
		name {processingUnitCost}
		type {int}
		default {10}
		desc { number of CPU cycles needed to fire which is scalable by the amount of data }
		attributes { A_CONSTANT | A_SETTABLE }
	}
	defstate {
		name {memoryCostOverhead}
		type {int}
		default {20}
		desc { bytes of overhead memory needed to fire }
		attributes { A_CONSTANT | A_SETTABLE }
	}
	defstate {
		name {memoryUnitCost}
		type {int}
		default {8}
		desc { bytes of overhead memory needed to fire which is dependent of the amount of data }
		attributes { A_CONSTANT | A_SETTABLE }
	}
	defstate {
		name {vectorizable}
		type {int}
		default {1}
		desc { specifies whether or not function can process more than one data particle per firing }
		attributes { A_CONSTANT | A_SETTABLE }
	}
	defstate {
		name {processorMapping}
		type {intarray}
		default {0}
		desc { List of processors (by number) of the function blocks mapped to outputPort }
		attributes { A_CONSTANT | A_SETTABLE }
	}
	defstate {
		name {functionBlockMapping}
		type {intarray}
		default {0}
		desc { List of function blocks (by number) mapped to outputPort }
		attributes { A_CONSTANT | A_SETTABLE }
	}
	defstate {
		name {inputPortMapping}
		type {intarray}
		default {0}
		desc { List of inputPorts (by number) of the functional blocks mapped to outputPort }
		attributes { A_CONSTANT | A_SETTABLE }
	}
    	header {
	  typedef char * CharPtr;
	}
	protected {
	  int i;
	  int* queuedCount;
	  int awaitingAcknowledge;
	  InDEPort* source;
	  OutDEPort* dest;
	  int vector, temp;
	  int outSize;
	  int processingCost, memoryCost;
	  int* inSamples;
	  int* outSamples;
	  CharPtr *inDataType;
	  CharPtr *outDataType;
	  ofstream clog;
	}
	hinclude { <iostream.h>, <fstream.h>, <string.h>, "Message.h", "DataBlock.h", "ResourceBlock.h" }
	ccinclude { "DEProcessor.h" }
	constructor {
	  delayType = TRUE;
	  awaitingAcknowledge = 0;
	}
	begin {
	  queuedCount = new int[inputPort.numberPorts()];
	  for (i = 0; i < inputPort.numberPorts(); i++)
	    queuedCount[i] = 0;
     	  clog.open(DEProcessor::logname, ios::app);
          clog << this->fullName() << ": inputPorts = " <<  inputPort.numberPorts() << ", outputPorts = " << outputPort.numberPorts() << endl;
	  clog.close();

	  inSamples = new int[inputPort.numberPorts()];
	  if (inputSamples.size() == 1)
	    for (i = 0; i < inputPort.numberPorts(); i++)
	      inSamples[i] = inputSamples[0];
	  else
	    {
	    if (inputSamples.size() != inputPort.numberPorts())
	      {
	      cerr << "Fatal Error: " << this->fullName() << ", number of inputSamples specified does not match number of inputPorts" << endl;
	      Error::abortRun(*this, "Number of inputSamples specified does not match number of inputPorts");
	      }	      
	    for (i = 0; i < inputPort.numberPorts(); i++)
	      inSamples[i] = inputSamples[i];
	    }

	  outSamples = new int[outputPort.numberPorts()];
	  if (outputSamples.size() == 1)
	    for (i = 0; i < outputPort.numberPorts(); i++)
	      outSamples[i] = outputSamples[0];
	  else
	    {
	    if (outputSamples.size() != outputPort.numberPorts())
	      {
	      cerr << "Fatal Error: " << this->fullName() << ", number of outputSamples specified does not match number of outputPorts" << endl;
	      Error::abortRun(*this, "Number of outputSamples specified does not match number of outputPorts");
	      }
	    for (i = 0; i < outputPort.numberPorts(); i++)
	      outSamples[i] = outputSamples[i];
	    }

	  inDataType = new CharPtr[inputPort.numberPorts()];
	  if (inputDataType.size() == 1)
	    for (i = 0; i < inputPort.numberPorts(); i++)
	      {
	      inDataType[i] = new char[strlen(inputDataType[0])];
	      strcpy(inDataType[i], inputDataType[0]);
	      }
	  else
	    {
	    if (inputDataType.size() != inputPort.numberPorts())
	      {
	      cerr << "Fatal Error: " << this->fullName() << ", number of inputDataTypes specified does not match number of inputPorts" << endl;
	      Error::abortRun(*this, "Number of inputDataTypes specified does not match number of inputPorts");
	      }	      
	    for (i = 0; i < inputPort.numberPorts(); i++)
	      {
	      inDataType[i] = new char[strlen(inputDataType[i])];
	      strcpy(inDataType[i], inputDataType[i]);
	      }
	    }


	  outDataType = new CharPtr[outputPort.numberPorts()];
	  if (outputDataType.size() == 1)
	    for (i = 0; i < outputPort.numberPorts(); i++)
	      {
	      outDataType[i] = new char[strlen(outputDataType[0])];
	      strcpy(outDataType[i], outputDataType[0]);
	      }
	  else
	    {
	    if (outputDataType.size() != outputPort.numberPorts())
	      {
	      cerr << "Fatal Error: " << this->fullName() << ", number of outputDataTypes specified does not match number of outputPorts" << endl;
	      Error::abortRun(*this, "Number of outputDataTypes specified does not match number of outputPorts");
	      }	      
	    for (i = 0; i < outputPort.numberPorts(); i++)
	      {
	      outDataType[i] = new char[strlen(outputDataType[i])];
	      strcpy(outDataType[i], outputDataType[i]);
	      }
	    }
	}
	go {
    	  // check to see if acknowledge arrived
	  if (acknowledge.dataNew)
	    {
	    Envelope pkt;
	    (acknowledge.get()).getMessage(pkt);
	    if (!pkt.typeCheck("ResourceBlock"))
	      {
	      cerr << "Fatal Error: " << this->fullName() << ", typeError(ResourceBlock)" << endl;
	      Error::abortRun(*this, pkt.typeError("ResourceBlock"));
	      return;
	      }
            const ResourceBlock* resourceBlockPtr = (const ResourceBlock *)pkt.myData();
	    completionTime = arrivalTime;
     	    clog.open(DEProcessor::logname, ios::app);
	    clog << this->fullName() << ", acknowledge to process " << resourceBlockPtr->vectorSize();
	    clog << " data blocks (particles) received at time " << arrivalTime << endl;
	    clog.close();

	    // "consume" DataBlocks on input queues
	    for (i = 0; i < inputPort.numberPorts(); i++)
	      queuedCount[i] -= inSamples[i];

	    // send data out each outputPort, scaled by the vectorSize
	    OutDEMPHIter destIter(outputPort);
	    DataBlock* dataBlockPtr;
	    for (i = 0; i < outputPort.numberPorts(); i++)
	      {
	      dest = destIter++;
 	      dataBlockPtr = new DataBlock((resourceBlockPtr->vectorSize() * outSamples[i]), outDataType[i],
                                              processorMapping[i], functionBlockMapping[i], inputPortMapping[i]);
	      Envelope pkt(*dataBlockPtr);
	      dest->put(completionTime) << pkt;
     	      clog.open(DEProcessor::logname, ios::app);
	      clog << this->fullName() << ", a data block of " << dataBlockPtr->dataSize() << " samples of ";
              clog << dataBlockPtr->dataRepresentation() << " sent out outputPort " << i << " to functionBlock ";
	      clog << functionBlockMapping[i] << " (inputPort " << inputPortMapping[i] << ") on processor ";
	      clog << processorMapping[i] << " at time " << completionTime << endl;
	      clog.close();
	      }

	    // no longer waiting for an acknowledge
	    awaitingAcknowledge = 0;
	    }

	  // else data has arrived on one or more inputPort
          InDEMPHIter sourceIter(inputPort);
	  for (i = 0; i < inputPort.numberPorts(); i++) {
	    source = sourceIter++;
	    if (source->dataNew)
	      {
	      Envelope pkt;
	      (source->get()).getMessage(pkt);
	      if (!pkt.typeCheck("DataBlock"))
	        {
	        cerr << "Fatal Error: " << this->fullName() << ", typeError(DataBlock)" << endl;
	        Error::abortRun(*this, pkt.typeError("DataBlock"));
		return;
	        }
              const DataBlock* dataBlockPtr = (const DataBlock *)pkt.myData();
	      if (dataBlockPtr->functionBlock() != functionNumber)
	        {
	        cerr << "Fatal Error: " << this->fullName() << ", Incorrectly addressed DataBlock received (wrong function)" << endl;
	        Error::abortRun(*this,"Incorrectly addressed DataBlock received (wrong function)");
	        return;
	        }
	      if ((dataBlockPtr->portList(0) < 0) || (dataBlockPtr->portList(0) >= inputPort.numberPorts()))
	        {
	        cerr << "Fatal Error: " << this->fullName() << ", Incorrectly addressed DataBlock received (right function, bad port)" << endl;
	        Error::abortRun(*this,"Incorrectly addressed DataBlock received (right function, bad port)");
	        return;
	        }
	      if (strcmp(dataBlockPtr->dataRepresentation(), inDataType[i]) != 0)
	        {
	        cerr << "Fatal Error: " << this->fullName() << ", DataBlock with wrong data type received" << endl;
	        Error::abortRun(*this,"DataBlock with wrong data type received");
	        return;
	        }
	      queuedCount[dataBlockPtr->portList(0)] += inSamples[i];
	      // temporary assumption, assumes all blocks in and out are same size
	      outSize = dataBlockPtr->dataSize();
     	      clog.open(DEProcessor::logname, ios::app);
	      clog << this->fullName() << ", a data block of " << dataBlockPtr->dataSize() << " samples of ";
	      clog << dataBlockPtr->dataRepresentation() << " arrived on inputPort " << dataBlockPtr->portList(0) << " at time " << arrivalTime << endl;
	      clog.close();
	      }
	  }	    

	  // exit if there is not enough DataBlocks on all ports
	  for (i = 0; i < inputPort.numberPorts(); i++)
	    if (queuedCount[i] < inSamples[i])
	      return;

	  // exit if waiting for an acknowledge
	  if (awaitingAcknowledge)
	      return;

	  // enough data is now available on all input ports to fire

/*
	  if (vectorizable)
	    {
	    // check for vectorization
	    vector = queuedCount[0] / inSamples[0];
	    for (i = 1; i < inputPort.numberPorts(); i++)
	      {
	      // calculate number of firings that can be combined (integer division)
	      temp = queuedCount[i] / inSamples[i];
	      if (temp < vector)
	        vector = temp;
	      }
	    }
	  else
	    // else process one data block (particle)
*/
	  // currently not used
	  vector = 1;

	  // send request for processor for processing and memory resources
	  processingCost = int(processingCostOverhead) + int(processingUnitCost) * vector;
	  memoryCost = int(memoryCostOverhead) + int(memoryUnitCost) * vector;
	  ResourceBlock* resourceBlockPtr = new ResourceBlock(functionName, functionNumber, processingCost, memoryCost, outSize);
	  Envelope pkt(*resourceBlockPtr);
	  completionTime = arrivalTime;
	  request.put(completionTime) << pkt;
	  awaitingAcknowledge = 1;
     	  clog.open(DEProcessor::logname, ios::app);
	  clog << this->fullName() << ", request to process " << vector << " data blocks (particles) sent at time " << completionTime << endl;
	  clog.close();
	}
}
