defstar {
	name {Processor}
	domain {DE}
	version { $Revision: 1.10 $ $Date: 1995/07/17 23:47:36 $ }
	author { E. K. Pauer }
	copyright {
Copyright (c) 1995 Sanders, a Lockheed Martin Company
All rights reserved.
	}
	acknowledge {
This work was performed by Sanders, a Lockheed Martin Company, as part
of the Sanders RASSP program under contract N00014-93-C-2172 to the
Naval Research Laboratory, 4555 Overlook Avenue, SW, Washington, DC
20375-5326. The Sponsoring Agency is: Advanced Research Projects
Agency, Electronic System Technology Office, 3701 North Fairfax
Drive, Arlington, VA 22203-1714. The Sanders RASSP team consists of
Sanders, Motorola, Hughes, and ISX.
}
	location { $PTOLEMY/src/contrib/atrade/de/stars }
	desc { Models the a processor by coordinating resources (CPU and memory). }
	input {
		name {externalInput}
		type {message}
	}
	output {
		name {externalOutput}
		type {message}
	}
	inmulti {
		name {internalInput}
		type {message}
	}
	outmulti {
		name {internalOutput}
		type {message}
	}
	inmulti {
		name {request}
		type {message}
	}
	outmulti {
		name {acknowledge}
		type {message}
	}
	defstate {
		name {partNumber}
		type {string}
		default {""}
		desc { Type of processor by part number }
		attributes { A_CONSTANT | A_SETTABLE }
	}
	defstate {
		name {processorNumber}
		type {int}
		default {0}
		desc { Unique id of the processor }
		attributes { A_CONSTANT | A_SETTABLE }
	}
	defstate {
		name {clockRate}
		type {int}
		default {1000000}
		desc { Clock rate of the processor }
		attributes { A_CONSTANT | A_SETTABLE }
	}
	defstate {
		name {inputSampleTransferRate}
		type {int}
		default {10000000}
		desc { Transfer rate of data at input }
		attributes { A_CONSTANT | A_SETTABLE }
	}
	defstate {
		name {outputSampleTransferRate}
		type {int}
		default {10000000}
		desc { Transfer rate of data at output }
		attributes { A_CONSTANT | A_SETTABLE }
	}
	defstate {
		name {functionBlockMapping}
		type {intarray}
		default {0}
		desc { List of function blocks (by number) mapped to internalOutput }
		attributes { A_CONSTANT | A_SETTABLE }
	}
	defstate {
		name {inputPortMapping}
		type {intarray}
		default {0}
		desc { List of input ports (by number) of the functional blocks mapped to internalOutput }
		attributes { A_CONSTANT | A_SETTABLE }
	}
	hinclude { <iostream.h>, <fstream.h>, "Message.h", "DataBlock.h", "ResourceBlock.h" }
	constructor {
	  delayType = TRUE;
	}
	protected {
	  int portNumberIndex;
	  int found;
	  InDEPort* source;
	  OutDEPort* dest;
	  double delay;
	  int i, j;
	  ofstream clog;
	}
        public {
	  static const char* filename;
	  static const char* logname;
	}
        code {
          const char* DEProcessor::filename = "/usr/tmp/gantt.dat";
	  const char* DEProcessor::logname = "/usr/tmp/de_log.dat";
        }
	begin {
     	  clog.open(logname, ios::app);
          clog << this->fullName() << ": internalInputs = " << internalInput.numberPorts();
	  clog << ", internalOutputs = " << internalOutput.numberPorts();
	  clog << ", requests = " << request.numberPorts() << ", acknowledges = " << acknowledge.numberPorts() << endl;
	  clog.close();

	  // check to insure there are consistent number of ports
	  if (request.numberPorts() != acknowledge.numberPorts())
	    {
	    cerr << "Fatal Error: " << this->fullName() << ", Inconsistent number of request and acknowledge ports" << endl;
	    Error::abortRun(*this,"Inconsistent number of request and acknowledge ports");
	    }
	  // check to insure the correct number of mappings have been specified
	  if (internalOutput.numberPorts() != functionBlockMapping.size())
	    {
	    cerr << "Fatal Error: " << this->fullName() << ", Incorrect number of function block mappings specified for internalInput" << endl;
	    Error::abortRun(*this,"Incorrect number of function block mappings specified for internalInput");
	    }
	  if (internalOutput.numberPorts() != inputPortMapping.size())
	    {
	    cerr << "Fatal Error: " << this->fullName() << ", Incorrect number of inputPort mappings specified for internalInput" << endl;
	    Error::abortRun(*this,"Incorrect number of inputPort mappings specified for internalInput");
	    }
	}
	go
	  {
	  // check if data has arrived on the externalInput
	  if (externalInput.dataNew)
	    {
	    Envelope pkt;
	    (externalInput.get()).getMessage(pkt);
	    if (!pkt.typeCheck("DataBlock"))
	      {
	      cerr << "Fatal Error: " << this->fullName() << ", typeError(DataBlock)" << endl;
	      Error::abortRun(*this, pkt.typeError("DataBlock"));
	      return;
	      }
            const DataBlock* dataBlockPtr = (const DataBlock *)pkt.myData();
	    if (dataBlockPtr->processor() != processorNumber)
	      {
     	      clog.open(logname, ios::app);
	      clog << this->fullName() << ", a data block of " << dataBlockPtr->dataSize() << " samples of ";
	      clog << dataBlockPtr->dataRepresentation() << " did not match this processor and has been ignored" << endl;
	      clog.close();
	      }
	    else
	      {
	      // calculate delay
	      delay = double(dataBlockPtr->dataSize()) / double(inputSampleTransferRate);
     	      clog.open(logname, ios::app);
	      clog << this->fullName() << ", a data block of " << dataBlockPtr->dataSize() << " samples of ";
	      clog << dataBlockPtr->dataRepresentation() << " arrived on externalInput at time " << arrivalTime;
	      clog << ", transfer time of " << delay << " calculated" << endl;
	      clog.close();
	      if (arrivalTime > completionTime)
	        completionTime = arrivalTime + delay;
	      else
	        completionTime += delay;
	      ofstream outFile(filename, ios::app);
	      if (!outFile)
	        {
	        cerr << "Fatal Error: " << this->fullName() << ", unable to open output file " << filename << endl;
	        Error::abortRun(*this,"Unable to open output file.");
	        }
	      outFile << "P" << processorNumber << " t " << (completionTime - delay) << " " << completionTime << endl;
	      outFile.close();
	      found = 0;
	      // attempt to find index to portNumber in logical list for internalOutput
              for (portNumberIndex = 0; portNumberIndex < internalOutput.numberPorts(); portNumberIndex++)
	        {
	        if ((dataBlockPtr->functionBlock() == functionBlockMapping[portNumberIndex]) &&
		    (dataBlockPtr->portList(0) == inputPortMapping[portNumberIndex]))
		  {
		  found = 1;
		  break;
		  }
	        }
	      if (found)
	        {
	        // find correct port to send data using index
                OutDEMPHIter destIter1(internalOutput);
	        for (j = 0; j <= portNumberIndex; j++)
		  dest = destIter1++;
	        // send data to appropriate feedOutput
	        dest->put(completionTime) << pkt;
     	        clog.open(logname, ios::app);
	        clog << this->fullName() << ", a data block of " << dataBlockPtr->dataSize() << " samples of ";
                clog << dataBlockPtr->dataRepresentation() << " sent out internalOutput ";
	        clog << portNumberIndex << " to functionBlock " << functionBlockMapping[portNumberIndex];
	        clog << " (inputPort " << inputPortMapping[portNumberIndex] << ") at time " << completionTime << endl;
		clog.close();
	        }
	      else
	        {
	        cerr << "Fatal Error: " << this->fullName() << ", Incorrectly addressed DataBlock received on externalInput" << endl;
	        Error::abortRun(*this,"Incorrectly addressed DataBlock received on externalInput");
		return;
	        }
	      }
	    }

	  // check if data has arrived on the internalInput
          InDEMPHIter sourceIter2(internalInput);
	  for (i = 0; i < internalInput.numberPorts(); i++)
	    {
	    source = sourceIter2++;
	    if (source->dataNew)
	      {
	      Envelope pkt;
	      (source->get()).getMessage(pkt);
	      if (!pkt.typeCheck("DataBlock"))
	        {
	        cerr << "Fatal Error: " << this->fullName() << ", typeError(DataBlock)" << endl;
	        Error::abortRun(*this, pkt.typeError("DataBlock"));
	        return;
	        }
              const DataBlock* dataBlockPtr = (const DataBlock *)pkt.myData();
	      // if data is not for this processor, send it out the externalOutput
	      if (dataBlockPtr->processor() != processorNumber)
	        {
	        delay = double(dataBlockPtr->dataSize()) / double(outputSampleTransferRate);
		clog.open(logname, ios::app);
	        clog << this->fullName() << ", a data block of " << dataBlockPtr->dataSize() << " samples of ";
		clog << dataBlockPtr->dataRepresentation() << " arrived on internalInput " << i << " at time " << arrivalTime;
	        clog << ", transfer time of " << delay << " calculated" << endl;
		clog.close();
	        if (arrivalTime > completionTime)
	          completionTime = arrivalTime + delay;
	        else
	          completionTime += delay;
	        ofstream outFile(filename, ios::app);
	        if (!outFile)
	          {
	          cerr << "Fatal Error: " << this->fullName() << ", unable to open output file " << filename << endl;
	          Error::abortRun(*this,"Unable to open output file.");
	          }
	        outFile << "P" << processorNumber << " t " << (completionTime - delay) << " " << completionTime << endl;
	        outFile.close();
		externalOutput.put(completionTime) << pkt;
     	        clog.open(DEProcessor::logname, ios::app);
		clog << this->fullName() << ", a data block of " << dataBlockPtr->dataSize() << " samples of ";
		clog << dataBlockPtr->dataRepresentation() << " sent out externalOutput at time " << completionTime << endl;
		clog.close();
	        }
	      // else data is for this processor, send out appropriate internalOutput port
	      else
		{
		found = 0;
	        // attempt to find index to portNumber in logical list for internalOutput
                for (portNumberIndex = 0; portNumberIndex < internalOutput.numberPorts(); portNumberIndex++)
	          if ((dataBlockPtr->portList(0) == inputPortMapping[portNumberIndex]) &&
		      (dataBlockPtr->functionBlock() == functionBlockMapping[portNumberIndex]))
		    {
		    found = 1;
		    break;
		    }
	        if (found)
	          {
	      	  // find correct port to send data using index
              	  OutDEMPHIter destIter2(internalOutput);
	      	  for (j = 0; j <= portNumberIndex; j++)
		    dest = destIter2++;
	      	  // send data to appropriate internalOutput, no delay incurred when passing data between functions on same processor
	      	  dest->put(arrivalTime) << pkt;
		  clog.open(logname, ios::app);
	          clog << this->fullName() << ", a data block of " << dataBlockPtr->dataSize() << " samples of ";
                  clog << dataBlockPtr->dataRepresentation() << " sent out internalOutput " << portNumberIndex;
		  clog << " to functionBlock " << functionBlockMapping[portNumberIndex];
	          clog << " (inputPort " << inputPortMapping[portNumberIndex] << ") on processor ";
		  clog << dataBlockPtr->processor() << " at time " << arrivalTime << endl;	
		  clog.close();
      	  	  }
	    	else
		  {
	          cerr << "Fatal Error: " << this->fullName() << ", Incorrectly addressed DataBlock received on internalInput" << endl;
		  Error::abortRun(*this, "Incorrectly addressed DataBlock received on internalInput");
		  return;
		  }
	        }
	      }
	    }

	  // Handle any requests for resources that have arrived
          InDEMPHIter sourceIter3(request);
          OutDEMPHIter destIter3(acknowledge);
	  for (i = 0; i < request.numberPorts(); i++)
	    {
	    source = sourceIter3++;
	    dest = destIter3++;
	    if (source->dataNew)
	      {
	      Envelope pkt;
      	      (source->get()).getMessage(pkt);
	      if (!pkt.typeCheck("ResourceBlock"))
	        {
	        cerr << "Fatal Error: " << this->fullName() << ", typeError(ResourceBlock)" << endl;
	        Error::abortRun(*this, pkt.typeError("ResourceBlock"));
	        return;
	        }
              const ResourceBlock* resourceBlockPtr = (const ResourceBlock *)pkt.myData();
	      delay = double(resourceBlockPtr->cycles()) / double(clockRate);
	      clog.open(logname, ios::app);
	      clog << this->fullName() << ", request to process " << resourceBlockPtr->vectorSize();
	      clog << " data blocks (particles) (" << resourceBlockPtr->cycles() << " cycles, " << resourceBlockPtr->memory();
	      clog << " bytes) arrived on request " << i << " at time " << arrivalTime;
	      clog << ", processing time of " << delay << " calculated" << endl;
	      clog << this->fullName() << ", Current completionTime = " << completionTime;
	      clog << ", Current arrivalTime = " << arrivalTime;
	      clog << ", Current delay = " << delay << endl;
	      clog.close();
	      if (arrivalTime > completionTime)
		completionTime = arrivalTime + delay;
	      else
		completionTime += delay;
	      ofstream outFile(filename, ios::app);
	      if (!outFile)
		{
	        cerr << "Fatal Error: " << this->fullName() << ", unable to open output file " << filename << endl;
	  	Error::abortRun(*this,"Unable to open output file.");
		}
	      outFile << "P" << processorNumber << " " << resourceBlockPtr->functionName() << " " << (completionTime - delay) << " " << completionTime << endl;
	      outFile.close();
	      // send message out acknowledge port
	      dest->put(completionTime) << pkt;
	      clog.open(DEProcessor::logname, ios::app);
	      clog << this->fullName() << ", acknowledge to process " << resourceBlockPtr->vectorSize();
	      clog << " particles (" << resourceBlockPtr->cycles() << " cycles, " << resourceBlockPtr->memory();
	      clog << " bytes) sent on acknowledge " << i << " at time " << completionTime << endl;
	      clog.close();
	      }
	    }	    
	}
}
