defstar {
	name {Source}
	derivedFrom { RepeatStar }
	domain {DE}
	version { $Revision: 1.5 $ $Date: 1995/07/17 23:47:11 $ }
	author { E. K. Pauer }
	copyright {
Copyright (c) 1995 Sanders, a Lockheed Martin Company
All rights reserved.
	}
	acknowledge {
This work was performed by Sanders, a Lockheed Martin Company, as part
of the Sanders RASSP program under contract N00014-93-C-2172 to the
Naval Research Laboratory, 4555 Overlook Avenue, SW, Washington, DC
20375-5326. The Sponsoring Agency is: Advanced Research Projects
Agency, Electronic System Technology Office, 3701 North Fairfax
Drive, Arlington, VA 22203-1714. The Sanders RASSP team consists of
Sanders, Motorola, Hughes, and ISX.
}
	location { $PTOLEMY/src/contrib/atrade/de/stars }
	desc { Creates DataBlock particles at a periodic rate. }
	output {
		name {outputPort}
		type {message}
	}
	defstate {
		name {processorMapping}
		type {int}
		default {0}
		desc { Processor to which DataBlocks will be routed }
		attributes { A_CONSTANT | A_SETTABLE }
	}
	defstate {
		name {functionBlockMapping}
		type {int}
		default {0}
		desc { FunctionBlock to which DataBlocks will be routed }
		attributes { A_CONSTANT | A_SETTABLE }
	}
	defstate {
		name {inputPortMapping}
		type {int}
		default {0}
		desc { Port on FunctionBlock to which DataBlocks will be routed }
		attributes { A_CONSTANT | A_SETTABLE }
	}
	defstate {
		name {dataType}
		type {string}
		default {"FLOAT"}
		desc { Type of data which DataBlock will carry }
		attributes { A_CONSTANT | A_SETTABLE }
	}
	defstate {
		name {blockSize}
		type {int}
		default {1}
		desc { Size of DataBlocks that are generated }
		attributes { A_CONSTANT | A_SETTABLE }
	}
	defstate {
		name {frequency}
		type {float}
		default {0.0}
		desc { Frequency at which DataBlocks will be generated }
		attributes { A_CONSTANT | A_SETTABLE }
	}
	defstate {
		name {delay}
		type {float}
		default {0.0}
		desc { Initial delay before generating DataBlocks }
		attributes { A_CONSTANT | A_SETTABLE }
	}
		
	hinclude { <iostream.h>, <fstream.h>, "Message.h", "DataBlock.h", <string.h> }
        ccinclude { "DEProcessor.h" }
	protected {
          DataBlock *pd;
	  double interval;
	  ofstream clog;
	}
	setup {
	  if ( !strcmp(dataType, INT) && !strcmp(dataType, FLOAT) &&
               !strcmp(dataType, COMPLEX) && !strcmp(dataType, FIX))
	    {
	    cerr << "Fatal Error: " << this->fullName() << ", Illegal type for data representation" << endl;
	    Error::abortRun(*this, "Illegal type for data representation");
	    }
	  if (frequency > 0.0)
	    interval = 1.0 / double(frequency);
	  else
            interval = 0.0;
	}
	go {
	  // add initial delay on first firing
	  if (completionTime == 0.0)
	    completionTime += delay;

	  // create DataBlock particle
	  pd = new DataBlock(blockSize, dataType, processorMapping, functionBlockMapping, inputPortMapping);
	  Envelope pkt(*pd);
	  outputPort.put(completionTime) << pkt;

     	  clog.open(DEProcessor::logname, ios::app);
	  clog << this->fullName() << ", a data block of " << blockSize << " samples of ";
	  clog << dataType  << " created and sent to functionBlock " << functionBlockMapping;
	  clog << " (inputPort " << inputPortMapping << ") on processor " << processorMapping;
	  clog << " at time " << completionTime << endl;
	  clog.close();

	  // fire only once if frequency is zero
	  if (interval == 0.0)
	    return;

	  // schedule the next firing
	  refireAtTime(completionTime);
	  // increment completion time
	  completionTime += double(interval);
	}
}

