/***************************************************************************
 *
 *  File Name:                  Ptcl.cc
 *
 *  Statement of Function:      Description of Ptcl class
 *
 *
 * This work was performed by Sanders, a Lockheed Martin Company, as part
 * of the Sanders RASSP program under contract N00014-93-C-2172 to the
 * Naval Research Laboratory, 4555 Overlook Avenue, SW, Washington, DC
 * 20375-5326. The Sponsoring Agency is: Advanced Research Projects
 * Agency, Electronic System Technology Office, 3701 North Fairfax
 * Drive, Arlington, VA 22203-1714. The Sanders RASSP team consists of
 * Sanders, Motorola, Hughes, and ISX.
 ***************************************************************************/

#include <iostream.h>
#include <stdlib.h>
#include <rw/ctoken.h>

#include "Ptcl.h"
#include "PtclPort.h"

static char sccsid[] = "%W% %G%";

#define DEBUG 0

/***************************************************************************
 *  Constructor
 ***************************************************************************/
Ptcl::Ptcl()
:ptcl_pipe_(RWCString("ptcl"))
{
  // clean the pipe, this will read off the first ptcl prompt
  if(DEBUG) cout << "Ptcl going to skip the pipe clean..." << endl;
//  sleep(5);
  ptcl_pipe_.pipeRead();
  if(DEBUG) cout << "Ptcl pipe is clean" << endl;
}

/***************************************************************************
 *  Copy Constructor
 ***************************************************************************/
Ptcl::Ptcl(const Ptcl&)
{
  Trace trace("Ptcl::Ptcl(const Ptcl&)"); trace.noop();
  log << "Copy constructor for Ptcl is not yet implemented." << endl;
}

/***************************************************************************
 *  Destructor
 ***************************************************************************/
Ptcl::~Ptcl()
{
  Trace trace("Ptcl::~Ptcl()"); trace.noop();
}

/***************************************************************************
 *  Equality operator
 ***************************************************************************/
int Ptcl::operator ==(const Ptcl&)
{
  Trace trace("Ptcl::operator==(const Ptcl&)"); trace.noop();
  // Equality test goes here....
  log << "Equality operator for Ptcl is not yet implemented." << endl;
  return(1);
}

/***************************************************************************
 *  Assignment operator
 ***************************************************************************/
Ptcl& Ptcl::operator =(const Ptcl& rhs)
{
  Trace trace("Ptcl::operator=(const Ptcl&)"); trace.noop();
  if (this == &rhs) {	// Check for assignment to self
    return *this;
  }

  // Assignment stuff goes here....
  log << "Assignment operator for Ptcl is not yet implemented."<< endl;

  return *this;
}

/***************************************************************************
 *  Stream insertion operator
 ***************************************************************************/
ostream& operator <<(ostream& s, Ptcl&)
{
  Trace trace("Ptcl::operator<<()"); trace.noop();
  s << "<< operator for Ptcl is not yet implemented." << endl;

  return(s);
}

/***************************************************************************
 *  To satify compilers that sccsid is used.
 ***************************************************************************/
void Ptcl::sccs_noop()
{
  sccsid[0] = sccsid[0];
}

/***************************************************************************
 * Method to get a list of domains from the kernel
 ***************************************************************************/
RWCString Ptcl::makeCall(const RWCString& message)
{
  // Send and receive a message from the pipe
  RWCString return_message = ptcl_pipe_.writeRead(message);

  // If the pipe wrote someting back which contains \n
  if(return_message.contains('\n'))
    // Remove the trailing \n ptcl>
    return_message.remove(return_message.last('\n'));
  if(return_message == "ptcl> ")
    return_message.remove(0);

  cout.flush();

  return return_message;
}

/***************************************************************************
 * Method to get a list of domains from the kernel
 ***************************************************************************/
int Ptcl::domains(RWDlistCollectables& domain_list)
{
  RWCString string;

  RWCString message("domains\n");

  RWCString domain_names(makeCall(message));
  
  RWCTokenizer next(domain_names);

  while( !(string = next(" ")).isNull() )
    {
      RWCollectableString* new_string = new RWCollectableString(string);
      domain_list.insert(new_string);
    }

  return domain_list.entries();
}

/***************************************************************************
 * Method to get a list of domains from the kernel
 ***************************************************************************/
void Ptcl::reset()
{
  RWCString message("reset\n");
  // Send and receive a message from the pipe
 // ptcl_pipe_.pipeWrite(message);
  makeCall(message);
}

/***************************************************************************
 * Method to get a list of the blocks in a domain
 ***************************************************************************/
int Ptcl::knownlist(const RWCString& domain,RWSortedVector& block_list)
{
  RWCString string;

  RWCString block_names = makeCall("knownlist " + domain + "\n");

  RWCTokenizer next(block_names);

  while( !(string = next(" ")).isNull() )
    {
      RWCollectableString* new_string = new RWCollectableString(string);
      block_list.insert(new_string);
    }

  return block_list.entries();
}

/***************************************************************************
 * 
 ***************************************************************************/
void Ptcl::fdCallback(XtPointer client_data, int* fd, XtInputId*)
{
  ((Ptcl*)client_data)->fdCallback(*fd);
}

/***************************************************************************
 *
 ***************************************************************************/
void Ptcl::fdCallback(const int&)
{

}

/***************************************************************************
 * Get a list of states, port and multi ports for a star
 ***************************************************************************/
void Ptcl::listobjs(RWDlistCollectables& states,
		    RWDlistCollectables& ports,
		    RWDlistCollectables& multi_ports,
		    const RWCString& block_name)
{
  RWCString string;
  
  RWCString state_names = makeCall("listobjs states " + block_name + "\n");  
  RWCTokenizer next_state(state_names);  
  while( !(string = next_state(" ")).isNull() )
    {
      RWCollectableString* new_string = new RWCollectableString(string);
      states.insert(new_string);
    }

  RWCString port_names = makeCall("listobjs ports " + block_name + "\n");  
  RWCTokenizer next_port(port_names);  
  while( !(string = next_port(" ")).isNull() )
    {
      RWCollectableString* new_string = new RWCollectableString(string);
      ports.insert(new_string);
    }

  RWCString mport_names = makeCall("listobjs multiports " + block_name + "\n");  
  RWCTokenizer next_mport(mport_names);  
  while( !(string = next_mport(" ")).isNull() )
    {
      RWCollectableString* new_string = new RWCollectableString(string);
      multi_ports.insert(new_string);
    }     
}

/***************************************************************************
 * Method to fill in the guts of a star block from the kernel
 ***************************************************************************/
void Ptcl::fillPtclStar(const RWCString& block_name,StarBlock* star)
{
  RWCString string;
  RWCString name;
  PtclPort* port;
  RWCString description;

  RWCString star_data = makeCall("print " + block_name + "\n");  

  if(DEBUG) cout << "Ptcl::fillPtclStar start data:\n" << star_data <<
    endl;

  RWCTokenizer next(star_data);
  
  while( !(string = next("\n")).isNull() )
    {
      if(string.contains("Star:"))
	 {
	   string.remove(0,string.first(':')+1);
	   name = string;
	   name = name.strip(RWCString::both,' ');

	   if(DEBUG) cout << "Ptcl::fillPtclStar Got name: " << name <<
	     endl;
	 }
      if(string.contains("Descriptor:"))
	 {
	   if(DEBUG) cout << "Found descriptor" << endl;
	   // Strip of the tag
	   string.remove(0,string.first(':')+1);

	   // Set the star name and start the description
	   star->name(name);
	   description.append(string + "\n");

	   // While still in the description part
	   while( !(string = next("\n")).isNull() )
	     {
	       // If the description section is over
	       if(string == "Ports:" || string == "MultiPorts:")
		 {
		   if(DEBUG) cout << "Starting ports" << endl;
		   break;
		 }

	       // Add to the description
	       description.append(string + "\n");
	     }

	   // Set the star description
	   star->description(description);		   
	 }
      if(string.contains("Ports:") && !string.contains("MultiPorts:"))
	 {
	   // In the port section
	   if(DEBUG) cout << "Found a port.." << endl;

	   // While still in the description part
	   while( !(string = next("\n")).isNull() )
	     {
	       // If the ports section is over
	       if(string.contains("States:") || string.contains("MultiPorts:"))
		 break;
	       // If this is an input port
	       else if(string.contains("Input "))
		 {
		   if(DEBUG) cout << "Adding input" << endl;
		   string = string.strip(RWCString::both);
		   string.remove(0,string.first(' ')+1);
   		   string = string.strip(RWCString::both);
		   port = new PtclPort(string,SINGLE);	   
		   star->addInport(port);
		 }
	       // If this is an output port
	       else if(string.contains("Output "))
		 {
		   if(DEBUG) cout << "Adding output" << endl;
		   string = string.strip(RWCString::both);
		   string.remove(0,string.first(' ')+1);
   		   string = string.strip(RWCString::both);
		   port = new PtclPort(string,SINGLE);	   
		   star->addOutport(port);
		 }
	     }
	 }
      if(string.contains("MultiPorts:"))
	{
	  // In the port section
	  
	  // While still in the description part
	  while( !(string = next("\n")).isNull() )
	    {
	      // If the ports section is over
	      if(string.contains("States:"))
		break;
	      // If this is an input port
	      else if(string.contains("Input "))
		{
		  string = string.strip(RWCString::both);
		  string.remove(0,string.first(' ')+1);
		  string = string.strip(RWCString::both);
		  port = new PtclPort(string,MULTI);	   
		  star->addInport(port);
		}
	      // If this is an output port
	      else if(string.contains("Output "))
		{
		  string = string.strip(RWCString::both);
		  string.remove(0,string.first(' ')+1);
		  string = string.strip(RWCString::both);
		  port = new PtclPort(string,MULTI);	   
		  star->addOutport(port);
		}
	    }
	}
    /*  else if(string.contains("Multi       Output PortHole:"))
	 {
	   string.remove(0,string.first('.')+1);
	   port = new PtclPort(string,MULTI);
	   
	   star->addOutport(port);
	 }
      else if(string.contains("Multi       Input PortHole:"))
	 {
	   string.remove(0,string.first('.')+1);
	   port = new PtclPort(string,MULTI);
	   
	   star->addInport(port);
	 }
      else if(string.contains("Output PortHole:"))
	 {
	   string.remove(0,string.first('.')+1);
	   port = new PtclPort(string,SINGLE);
	   star->addOutport(port);
	 }
      else if(string.contains("Input PortHole:"))
	 {
	   string.remove(0,string.first('.')+1);
	   port = new PtclPort(string,SINGLE);	   
	   star->addInport(port);
	   }*/
    }
}

/***************************************************************************
 * Method to run a string of ptcl commands
 ***************************************************************************/
RWCString Ptcl::runScript(const RWCString& script)
{
  RWCString string;
  RWCString return_string;

  RWCTokenizer line(script);  
  while( !(string = line("\n")).isNull() )
    {
      RWCString my_line(string + "\n");
      RWCString my_line2(string + "\n");

      RWCString results = makeCall(my_line2);

      return_string.append(results);
      string.remove(0);
    }

  return return_string;
}

/***************************************************************************
 * Method to run a string of ptcl commands
 ***************************************************************************/
void Ptcl::getStates(StarBlock* star)
{
  RWCString string;

  // Get a list of all the states
  RWCString message("listobjs states " + star->name() + "\n");

  RWCString state_names = makeCall(message);

  // Parse the list of states
  RWCTokenizer next_state(state_names);  
  while( !(string = next_state(" ")).isNull() )
    {
      string = string.strip(RWCString::both,' ');     

      // Get the value
      message = ("statevalue " + star->name() + " " + string + " initial\n");

      // Ask ptcl for the state current value
      RWCString results(makeCall(message));
      if(results.length() > 5000)
	results = "";

      results = results.strip(RWCString::both,' ');

      // Add the state to the star
      star->addState(string,results);
    }
}

/***************************************************************************
 *
 ***************************************************************************/
void Ptcl::fillPortSize(StarBlock* star)
{
  RWCString string;

  // Get a list of all the states
  RWCString message("print " + star->uniqueName() + "\n");
  RWCString data = makeCall(message);

  RWCString info;

  RWCTokenizer next(data);
  
  while( !(string = next("\n")).isNull() )
    {
      if( (string.contains("Output PortHole:") || 
	   string.contains("Input PortHole:")) &&
	  !string.contains("Multi"))
	{
	  // Get the name of the porthole
	  string.remove(0,string.last('.')+1);

	  if(string.contains("#"))
	    string.remove(string.last('#'));
  
	  // Get the token count
	  info = next("\n");
	  info.remove(0,info.last('=')+1);	  
	  info.remove(info.last(')'));	  
	  
	  info = info.strip(RWCString::both,' ');
	}
    }
}

/***************************************************************************
 *
 ***************************************************************************/
void Ptcl::setDomain(const RWCString& domain)
{
  RWCString message("domain " + domain + "\n");
  // Send and receive a message from the pipe
 // ptcl_pipe_.pipeWrite(message); 
  makeCall(message);
}

/***************************************************************************
 * Method to link in an object file
 ***************************************************************************/
void Ptcl::link(const RWCString& object)
{
  // Link some stars
  RWCString message("link " + object + "\n");  
  makeCall(message);
}
