/***************************************************************************
 *
 *  File Name:			Ptcl.h
 *
 *  Statement of Function: Ptcl is a interface to the ptolemy ptcl 
 *                         command line interface
 *
 * This work was performed by Sanders, a Lockheed Martin Company, as part
 * of the Sanders RASSP program under contract N00014-93-C-2172 to the
 * Naval Research Laboratory, 4555 Overlook Avenue, SW, Washington, DC
 * 20375-5326. The Sponsoring Agency is: Advanced Research Projects
 * Agency, Electronic System Technology Office, 3701 North Fairfax
 * Drive, Arlington, VA 22203-1714. The Sanders RASSP team consists of
 * Sanders, Motorola, Hughes, and ISX.
 ***************************************************************************/

class Ptcl;

#ifndef PTCL_H
#define PTCL_H

#include <X11/Intrinsic.h>
#include <rw/sortvec.h>
#include <rw/dlistcol.h>
#include <rw/cstring.h>
#include <rw/collstr.h>
#include "spLog.h"
#include "spTrace.h"
#include "myPipe.h"
#include "StarBlock.h"

class Ptcl
{
  /*************************
   * Stream output operator
   *************************/
  friend ostream& operator <<(ostream& s, Ptcl&);

private:

  // Pipe to ptolemy ptcl
  myPipe ptcl_pipe_;

public:
  /**************************
   * Constructors/Destructor
   **************************/
  Ptcl(); 				// Default constructor
  Ptcl(const Ptcl& );		// Copy constructor
  ~Ptcl();				// Destructor

  /************
   * Operators
   ************/
  int operator ==(const Ptcl& rhs);		// Equality operator
  Ptcl& operator =(const Ptcl& rhs);	// Assignment operator

  /********************
   * Virtual functions
   ********************/

  /**************************
   * Member access functions
   **************************/

  // Return the pipe file descriptor
  int readFD(){return ptcl_pipe_.readFD();}

  // Set a pipe callback
  static void fdCallback(XtPointer client_data, int* fd, XtInputId* id);
  void fdCallback(const int& fd);

  /**************************
   * ptcl commands
   **************************/

  // Get a RWCString list of all the domains in the kernal
  // It will return the number of domains found
  int domains(RWDlistCollectables& domain_list);

  // Get a RWCString list of all the domains in the kernal
  // It will return the number of domains found
  int knownlist(const RWCString& domain,RWSortedVector& block_list);

  // Get a list of states, port and multi ports for a star
  void listobjs(RWDlistCollectables& states,
		RWDlistCollectables& ports,
		RWDlistCollectables& multi_ports,
		const RWCString& block_name);


  // Reset the ptolemy universe
  void reset();

  // Set the domain of the current universe
  void setDomain(const RWCString& domain);

  // Get a list of states, port and multi ports for a star
  RWCString runScript(const RWCString& string);

  // Fill a star block from ptcl star information
  void fillPtclStar(const RWCString& block_name, StarBlock* star);

  // Get the states of a star block
  void getStates(StarBlock* star);

  // Fill the port sizes of a star block
  void fillPortSize(StarBlock* star);

  // Dynamicly link in a new star object
  void link(const RWCString& object);

  /************************
   * Non-virtual functions
   ************************/

/********************
 * Private functions
 ********************/
private:

  // Make a call the ptolemy ptcl
  RWCString makeCall(const RWCString& message);

  void sccs_noop();			// Defined for sccs info
};

#endif PTCL_H
