/***************************************************************************
 *
 *  File Name:                  myPipe.cc
 *
 *  Statement of Function:      Description of myPipe class
 *
 * This work was performed by Sanders, a Lockheed Martin Company, as part
 * of the Sanders RASSP program under contract N00014-93-C-2172 to the
 * Naval Research Laboratory, 4555 Overlook Avenue, SW, Washington, DC
 * 20375-5326. The Sponsoring Agency is: Advanced Research Projects
 * Agency, Electronic System Technology Office, 3701 North Fairfax
 * Drive, Arlington, VA 22203-1714. The Sanders RASSP team consists of
 * Sanders, Motorola, Hughes, and ISX.
 ***************************************************************************/

#include <stdlib.h>
#include <iostream.h>
#include <unistd.h>

#include "myPipe.h"

static char sccsid[] = "%W% %G%";

#define DEBUG 0

/***************************************************************************
 *  Constructor
 ***************************************************************************/
myPipe::myPipe()
{
  clearBuffers();
}

/***************************************************************************
 * 
 ***************************************************************************/
void myPipe::clearBuffers()
{
  int i;
  for(i=0;i<MAX_BUFF;i++)
    {
      read_buffer_ [i] = 0;
      write_buffer_[i] = 0;
    }
}

/***************************************************************************
 *  Constructor
 ***************************************************************************/
myPipe::myPipe(const RWCString& command)
{
  int pipe1[2];
  int pipe2[2];

  clearBuffers();

  // Create some pipes
  if( pipe(pipe1) == -1 || pipe(pipe2) == -1 )
    cerr << "Could not create pipes." << endl;

  // Fork
  childpid_ = fork();
  
  if( childpid_ < 0)
    cerr << "Could not fork." << endl;

  // If we are in the child
  if(childpid_ == 0)
    {
      // Close the pipe ends we don\'t need
      close(pipe1[1]);
      close(pipe2[0]);
      
      // Close stdin
      close(0);
      dup(pipe1[0]);
      
      // Close stdout
      close(1);      
      dup(pipe2[1]);
      
      execlp(command.data(),command.data(),NULL);
      cerr << "Could not exec process: " << command << endl;
      exit(-1);
    }
  else
    {
      // Close the pipe ends we don\'t need
      close(pipe1[0]);
      close(pipe2[1]);

      to_child_fd_   = pipe1[1];
      from_child_fd_ = pipe2[0];
    }

  sleep(1);
}

/***************************************************************************
 *  Copy Constructor
 ***************************************************************************/
myPipe::myPipe(const myPipe&)
{
  Trace trace("myPipe::myPipe(const myPipe&)"); trace.noop();
  log << "Copy constructor for myPipe is not yet implemented." << endl;
}

/***************************************************************************
 *  Destructor
 ***************************************************************************/
myPipe::~myPipe()
{
  Trace trace("myPipe::~myPipe()"); trace.noop();

  close(to_child_fd_);
  close(from_child_fd_);
}

/***************************************************************************
 *  Equality operator
 ***************************************************************************/
int myPipe::operator ==(const myPipe&)
{
  Trace trace("myPipe::operator==(const myPipe&)"); trace.noop();
  // Equality test goes here....
  log << "Equality operator for myPipe is not yet implemented." << endl;
  return(1);
}

/***************************************************************************
 *  Assignment operator
 ***************************************************************************/
myPipe& myPipe::operator =(const myPipe& rhs)
{
  Trace trace("myPipe::operator=(const myPipe&)"); trace.noop();
  if (this == &rhs) {	// Check for assignment to self
    return *this;
  }

  // Assignment stuff goes here....
  log << "Assignment operator for myPipe is not yet implemented."<< endl;

  return *this;
}

/***************************************************************************
 *  Stream insertion operator
 ***************************************************************************/
ostream& operator <<(ostream& s, myPipe&)
{
  Trace trace("myPipe::operator<<()"); trace.noop();
  s << "<< operator for myPipe is not yet implemented." << endl;

  return(s);
}

/***************************************************************************
 *  To satify compilers that sccsid is used.
 ***************************************************************************/
void myPipe::sccs_noop()
{
  sccsid[0] = sccsid[0];
}

/***************************************************************************
 * 
 ***************************************************************************/
RWCString myPipe::pipeRead()
{
  int nbytes;
  RWCString data("");
  if(DEBUG) cout << "myPipe::pipeRead" << endl;
  while( (nbytes = read(from_child_fd_,read_buffer_,MAX_BUFF)) != -1)
    {
      if(DEBUG) cout << "myPipe::pipeRead() reading.." << endl;

      data.append(read_buffer_,nbytes);      

      if(nbytes != MAX_BUFF)
	break;
    }
  if(DEBUG) cout << "myPipe::pipeRead done" << endl;
  return data;
}

/***************************************************************************
 * 
 ***************************************************************************/
void myPipe::pipeWrite(const RWCString& data)
{
  int nbytes = data.length();

  if(DEBUG) cout << "myPipe::pipeWrite write: " << data << endl;
  write(to_child_fd_,data.data(),nbytes);
  if(DEBUG) cout << "myPipe::pipeWrite done" << endl;
}

/***************************************************************************
 * 
 ***************************************************************************/
RWCString myPipe::writeRead(const RWCString& data)
{
  pipeWrite(data);
  return pipeRead();
}
