/***************************************************************************
 *
 *  File Name:			myPipe.h
 *
 *  Statement of Function:	Description of myPipe class
 *
 *  Description:
 *
 * This work was performed by Sanders, a Lockheed Martin Company, as part
 * of the Sanders RASSP program under contract N00014-93-C-2172 to the
 * Naval Research Laboratory, 4555 Overlook Avenue, SW, Washington, DC
 * 20375-5326. The Sponsoring Agency is: Advanced Research Projects
 * Agency, Electronic System Technology Office, 3701 North Fairfax
 * Drive, Arlington, VA 22203-1714. The Sanders RASSP team consists of
 * Sanders, Motorola, Hughes, and ISX.
 ***************************************************************************/

class myPipe;

#ifndef MY_PIPE_H
#define MY_PIPE_H

#include <rw/cstring.h>
#include "spLog.h"
#include "spTrace.h"

static const int MAX_BUFF = 512;

class myPipe
{
  /*************************
   * Stream output operator
   *************************/
  friend ostream& operator <<(ostream& s, myPipe&);

private:

  char read_buffer_ [MAX_BUFF];
  char write_buffer_[MAX_BUFF];

  int childpid_;

  int to_child_fd_;
  int from_child_fd_;


public:
  /**************************
   * Constructors/Destructor
   **************************/
  myPipe(); 				// Default constructor
  myPipe(const RWCString& command);

  myPipe(const myPipe& );		// Copy constructor
  ~myPipe();				// Destructor

  /************
   * Operators
   ************/
  int operator ==(const myPipe& rhs);		// Equality operator
  myPipe& operator =(const myPipe& rhs);	// Assignment operator

  /********************
   * Virtual functions
   ********************/

  /**************************
   * Member access functions
   **************************/

  int readFD(){return from_child_fd_;}

  // Methods to read and write from the pipe
  RWCString pipeRead();
  void pipeWrite(const RWCString& data);

  // Method to do a combo write/read
  RWCString writeRead(const RWCString& data);

  /************************
   * Non-virtual functions
   ************************/

/********************
 * Private functions
 ********************/
private:

  void clearBuffers();
  void sccs_noop();			// Defined for sccs info
};

#endif MY_PIPE_H
